@@ -20,7 +20,7 @@ enum EventType
         EVENT_SHUFFLE,
         EVENT_KEY_PRESS,
         EVENT_SHOWKEYBINDINGS,
-        EVENT_SHOWINFO,
+        EVENT_SHOWPLAYLIST,
         EVENT_SHOWSEARCH,
         EVENT_GOTOSONG,
         EVENT_GOTOBEGINNINGOFPLAYLIST,
@@ -35,7 +35,8 @@ enum EventType
         EVENT_NEXTPAGE,
         EVENT_PREVPAGE,
         EVENT_REMOVE,
-        EVENT_SEARCH
+        EVENT_SEARCH,
+        EVENT_TABNEXT
 };
 
 struct Event
