@@ -3,6 +3,7 @@
 
 #include <dirent.h>
 #include <errno.h>
+#include <libgen.h>
 #include <pwd.h>
 #include <regex.h>
 #include <stdbool.h>
@@ -22,7 +23,7 @@
 #endif
 
 #ifndef AUDIO_EXTENSIONS
-#define AUDIO_EXTENSIONS "\\.(m4a|aac|mp4|mp3|ogg|flac|wav|opus)$"
+#define AUDIO_EXTENSIONS "\\.(m4a|aac|mp3|ogg|flac|wav|opus)$"
 #endif
 
 enum SearchType
