@@ -45,6 +45,7 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
 #include <stdio.h>
 #include <string.h>
 #include <sys/param.h>
+#include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
 #include "cache.h"
@@ -60,6 +61,9 @@ OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE. */
 #include "soundcommon.h"
 #include "songloader.h"
 #include "utils.h"
+#ifdef USE_LIBNOTIFY
+#include "libnotify/notify.h"
+#endif
 
 // #define DEBUG 1
 #define MAX_TMP_SEQ_LEN 256 // Maximum length of temporary sequence buffer
@@ -80,6 +84,9 @@ bool startFromTop = false;
 bool exactSearch = false;
 AppSettings settings;
 int fuzzySearchThreshold = 2;
+int lastNotifiedId = -1;
+bool songWasRemoved = false;
+bool noPlaylist = false;
 
 bool isCooldownElapsed(int milliSeconds)
 {
@@ -143,6 +150,7 @@ struct Event processInput()
                     strcmp(seq + 1, settings.hardNextPage) == 0 || strcmp(seq + 1, settings.hardPrevPage) == 0)
                 {
                         keyReleased = 0;
+                        readInputSequence(tmpSeq, sizeof(tmpSeq)); // dummy read to prevent scrolling after key released
                         break;
                 }
 
@@ -161,12 +169,12 @@ struct Event processInput()
         {
                 if (strcmp(event.key, "\x7F") == 0 || strcmp(event.key, "\x08") == 0)
                 {
-                        removeFromSearchText(getLibrary());
+                        removeFromSearchText();
                         chosenSearchResultRow = 0;
                         fuzzySearch(getLibrary(), fuzzySearchThreshold);
                         event.type = EVENT_SEARCH;
                 }
-                else if (((strlen(event.key) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0))
+                else if (((strlen(event.key) == 1 && event.key[0] != '\033' && event.key[0] != '\n' && event.key[0] != '\t' && event.key[0] != '\r') || strcmp(event.key, " ") == 0 || (unsigned char)event.key[0] >= 0xC0))
                 {
                         addToSearchText(event.key);
                         chosenSearchResultRow = 0;
@@ -187,7 +195,6 @@ struct Event processInput()
                                       {settings.quit, EVENT_QUIT},
                                       {settings.hardQuit, EVENT_QUIT},
                                       {settings.toggleShuffle, EVENT_SHUFFLE},
-                                      {settings.toggleCovers, EVENT_TOGGLECOVERS},
                                       {settings.toggleVisualizer, EVENT_TOGGLEVISUALIZER},
                                       {settings.toggleAscii, EVENT_TOGGLEBLOCKS},
                                       {settings.switchNumberedSong, EVENT_GOTOSONG},
@@ -204,20 +211,27 @@ struct Event processInput()
                                       {settings.hardSwitchNumberedSong, EVENT_GOTOSONG},
                                       {settings.hardScrollUp, EVENT_SCROLLPREV},
                                       {settings.hardScrollDown, EVENT_SCROLLNEXT},
-                                      {settings.hardShowInfo, EVENT_SHOWINFO},
-                                      {settings.hardShowInfoAlt, EVENT_SHOWINFO},
-                                      {settings.hardShowSearch, EVENT_SHOWSEARCH},
+                                      {settings.hardShowPlaylist, EVENT_SHOWPLAYLIST},
+                                      {settings.hardShowPlaylistAlt, EVENT_SHOWPLAYLIST},
+                                      {settings.showPlaylistAlt, EVENT_SHOWPLAYLIST},
                                       {settings.hardShowKeys, EVENT_SHOWKEYBINDINGS},
                                       {settings.hardShowKeysAlt, EVENT_SHOWKEYBINDINGS},
+                                      {settings.showKeysAlt, EVENT_SHOWKEYBINDINGS},
                                       {settings.hardEndOfPlaylist, EVENT_GOTOENDOFPLAYLIST},
                                       {settings.hardShowTrack, EVENT_SHOWTRACK},
                                       {settings.hardShowTrackAlt, EVENT_SHOWTRACK},
+                                      {settings.showTrackAlt, EVENT_SHOWTRACK},
                                       {settings.hardShowLibrary, EVENT_SHOWLIBRARY},
                                       {settings.hardShowLibraryAlt, EVENT_SHOWLIBRARY},
+                                      {settings.showLibraryAlt, EVENT_SHOWLIBRARY},
+                                      {settings.hardShowSearch, EVENT_SHOWSEARCH},
                                       {settings.hardShowSearchAlt, EVENT_SHOWSEARCH},
+                                      {settings.showSearchAlt, EVENT_SHOWSEARCH},
                                       {settings.hardNextPage, EVENT_NEXTPAGE},
                                       {settings.hardPrevPage, EVENT_PREVPAGE},
-                                      {settings.hardRemove, EVENT_REMOVE}};
+                                      {settings.hardRemove, EVENT_REMOVE},
+                                      {settings.hardRemove2, EVENT_REMOVE},
+                                      {settings.tabNext, EVENT_TABNEXT}};
 
         int numKeyMappings = sizeof(keyMappings) / sizeof(EventMapping);
 
@@ -329,6 +343,7 @@ void setEndOfListReached()
         loadedNextSong = false;
         audioData.endOfListReached = true;
         usingSongDataA = false;
+        currentSong = NULL;
         audioData.currentFileIndex = 0;
         audioData.restart = true;
         loadingdata.loadA = true;
@@ -341,45 +356,47 @@ void setEndOfListReached()
         chosenRow = playlist.count - 1;
 }
 
-void finishLoading()
+void notifyMPRISSwitch(SongData *currentSongData)
 {
-        int maxNumTries = 20;
-        int numtries = 0;
+        if (currentSongData == NULL)
+                return;
 
-        while (!loadedNextSong && !loadingFailed && numtries < maxNumTries)
-        {
-                c_sleep(100);
-                numtries++;
-        }
+        gint64 length = getLengthInMicroSec(currentSongData->duration);
 
-        loadedNextSong = true;
+        // update mpris
+        emitMetadataChanged(
+            currentSongData->metadata->title,
+            currentSongData->metadata->artist,
+            currentSongData->metadata->album,
+            currentSongData->coverArtPath,
+            currentSongData->trackId != NULL ? currentSongData->trackId : "", currentSong,
+            length);
 }
 
-void resetTimeCount()
+void notifySongSwitch(SongData *currentSongData)
 {
-        elapsedSeconds = 0.0;
-        pauseSeconds = 0.0;
-        totalPauseSeconds = 0.0;
-}
-
-void notifySongSwitch()
-{
-        SongData *currentSongData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
-        bool isDeleted = (audioData.currentFileIndex == 0) ? userData.songdataADeleted == true : userData.songdataBDeleted == true;
-
-        if (isDeleted == false && currentSongData != NULL && currentSongData->hasErrors == 0 && currentSongData->metadata && strlen(currentSongData->metadata->title) > 0)
+        if (currentSongData != NULL && currentSongData->hasErrors == 0 && currentSongData->metadata && strlen(currentSongData->metadata->title) > 0)
         {
+#ifdef USE_LIBNOTIFY
                 displaySongNotification(currentSongData->metadata->artist, currentSongData->metadata->title, currentSongData->coverArtPath);
+#endif
 
-                gint64 length = getLengthInSec(currentSongData->duration);
-                // update mpris
-                emitMetadataChanged(
-                    currentSongData->metadata->title,
-                    currentSongData->metadata->artist,
-                    currentSongData->metadata->album,
-                    currentSongData->coverArtPath,
-                    currentSongData->trackId != NULL ? currentSongData->trackId : "", currentSong,
-                    length);
+                notifyMPRISSwitch(currentSongData);
+
+                lastNotifiedId = currentSong->id;
+        }
+}
+
+void determineSongAndNotify()
+{
+        SongData *currentSongData = NULL;
+
+        bool isDeleted = determineCurrentSongData(&currentSongData);
+
+        if (lastNotifiedId != currentSong->id)
+        {
+                if (!isDeleted)
+                        notifySongSwitch(currentSongData);
         }
 }
 
@@ -400,6 +417,13 @@ void refreshPlayer()
                 return;
         }
 
+        if (doNotifyMPRISSwitched)
+        {
+                doNotifyMPRISSwitched = false;
+
+                notifyMPRISSwitch(getCurrentSongData());
+        }
+
         if (shouldRefreshPlayer())
         {
                 printPlayer(getCurrentSongData(), elapsedSeconds, &settings);
@@ -408,6 +432,53 @@ void refreshPlayer()
         pthread_mutex_unlock(&switchMutex);
 }
 
+void resetListAfterDequeuingPlayingSong()
+{
+        if (lastPlayedId < 0)
+                return;
+
+        Node *node = findSelectedEntryById(&playlist, lastPlayedId);
+
+        if (currentSong == NULL && node == NULL)
+        {
+                stopPlayback();
+
+                loadedNextSong = false;
+                audioData.endOfListReached = true;
+                audioData.restart = true;
+
+                emitMetadataChanged("", "", "", "", "/org/mpris/MediaPlayer2/TrackList/NoTrack", NULL, 0);
+                emitPlaybackStoppedMpris();
+
+                pthread_mutex_lock(&dataSourceMutex);
+                cleanupPlaybackDevice();
+                pthread_mutex_unlock(&dataSourceMutex);
+
+                refresh = true;
+
+                switchAudioImplementation();
+
+                unloadSongA();
+                unloadSongB();
+                songWasRemoved = true;
+                userData.currentSongData = NULL;
+
+                audioData.currentFileIndex = 0;
+                audioData.restart = true;
+                waitingForNext = true;
+                startFromTop = true;
+                loadingdata.loadA = true;
+                usingSongDataA = false;
+
+                ma_data_source_uninit(&audioData);
+
+                audioData.switchFiles = false;
+
+                if (playlist.count == 0)
+                        songToStartFrom = NULL;
+        }
+}
+
 void handleGoToSong()
 {
         if (goingToSong)
@@ -437,6 +508,7 @@ void handleGoToSong()
                 pthread_mutex_lock(&(playlist.mutex));
 
                 enqueueSongs(getCurrentLibEntry());
+                resetListAfterDequeuingPlayingSong();
 
                 pthread_mutex_unlock(&(playlist.mutex));
         }
@@ -447,6 +519,7 @@ void handleGoToSong()
                 setChosenDir(getCurrentSearchEntry());
 
                 enqueueSongs(getCurrentSearchEntry());
+                resetListAfterDequeuingPlayingSong();
 
                 pthread_mutex_unlock(&(playlist.mutex));
         }
@@ -454,13 +527,41 @@ void handleGoToSong()
         {
                 if (digitsPressedCount == 0)
                 {
-                        loadedNextSong = true;
+                        if (isPaused() && currentSong != NULL && chosenNodeId == currentSong->id)
+                        {
+                                togglePause(&totalPauseSeconds, &pauseSeconds, &pause_time);
+                        }
+                        else
+                        {
+                                loadedNextSong = true;
 
-                        playlistNeedsUpdate = false;
-                        nextSongNeedsRebuilding = false;
+                                playlistNeedsUpdate = false;
+                                nextSongNeedsRebuilding = false;
 
-                        skipToSong(chosenNodeId);
-                        audioData.endOfListReached = false;
+                                unloadSongA();
+                                unloadSongB();
+
+                                usingSongDataA = false;
+                                audioData.currentFileIndex = 0;
+                                loadingdata.loadA = true;
+
+                                bool wasEndOfList = false;
+                                if (audioData.endOfListReached)
+                                        wasEndOfList = true;
+
+                                skipToSong(chosenNodeId, true);
+
+                                if ((songWasRemoved && currentSong != NULL))
+                                {
+                                        usingSongDataA = !usingSongDataA;
+                                        songWasRemoved = false;
+                                }
+
+                                if (wasEndOfList)
+                                        usingSongDataA = true;
+
+                                audioData.endOfListReached = false;
+                        }
                 }
                 else
                 {
@@ -523,14 +624,12 @@ void handleInput()
         case EVENT_TOGGLEREPEAT:
                 toggleRepeat();
                 break;
-        case EVENT_TOGGLECOVERS:
-                toggleCovers(&settings);
-                break;
         case EVENT_TOGGLEBLOCKS:
                 toggleBlocks(&settings);
                 break;
         case EVENT_SHUFFLE:
                 toggleShuffle();
+                emitShuffleChanged();
                 break;
         case EVENT_TOGGLE_PROFILE_COLORS:
                 toggleColors(&settings);
@@ -546,9 +645,11 @@ void handleInput()
                 break;
         case EVENT_VOLUME_UP:
                 adjustVolumePercent(5);
+                emitVolumeChanged();
                 break;
         case EVENT_VOLUME_DOWN:
                 adjustVolumePercent(-5);
+                emitVolumeChanged();
                 break;
         case EVENT_NEXT:
                 resetPlaylistDisplay = true;
@@ -576,7 +677,7 @@ void handleInput()
         case EVENT_SHOWKEYBINDINGS:
                 toggleShowKeyBindings();
                 break;
-        case EVENT_SHOWINFO:
+        case EVENT_SHOWPLAYLIST:
                 toggleShowPlaylist();
                 break;
         case EVENT_SHOWSEARCH:
@@ -593,10 +694,14 @@ void handleInput()
                 break;
         case EVENT_REMOVE:
                 handleRemove();
+                resetListAfterDequeuingPlayingSong();
                 break;
         case EVENT_SHOWTRACK:
                 showTrack();
                 break;
+        case EVENT_TABNEXT:
+                tabNext();
+                break;
         default:
                 fastForwarding = false;
                 rewinding = false;
@@ -656,7 +761,14 @@ void loadAudioData()
                         }
                         else if (waitingForNext)
                         {
-                                if (lastPlayedId >= 0)
+                                if (songToStartFrom != NULL)
+                                {
+                                        // Make sure it still exists in the playlist
+                                        findNodeInList(&playlist, songToStartFrom->id, &currentSong);
+
+                                        songToStartFrom = NULL;
+                                }
+                                else if (lastPlayedId >= 0)
                                 {
                                         currentSong = findSelectedEntryById(&playlist, lastPlayedId);
                                         if (currentSong != NULL && currentSong->next != NULL)
@@ -677,9 +789,18 @@ void loadAudioData()
                         audioData.restart = false;
                         waitingForPlaylist = false;
                         waitingForNext = false;
+                        songWasRemoved = false;
+
+                        if (isShuffleEnabled())
+                                reshufflePlaylist();
+
+                        unloadSongA();
+                        unloadSongB();
 
                         int res = loadFirst(currentSong);
 
+                        finishLoading();
+
                         if (res >= 0)
                         {
                                 res = createAudioDevice(&userData);
@@ -704,13 +825,8 @@ void loadAudioData()
         }
         else if (currentSong != NULL && (nextSongNeedsRebuilding || nextSong == NULL) && !songLoading)
         {
-                songLoading = true;
-                nextSongNeedsRebuilding = false;
-                tryNextSong = currentSong->next;
-                loadingdata.loadA = !usingSongDataA;
-                nextSong = getListNext(currentSong);
-                loadingdata.loadingFirstDecoder = false;
-                loadSong(nextSong, &loadingdata);
+                loadNextSong();
+                determineSongAndNotify();
         }
 
         loadingAudioData = false;
@@ -739,13 +855,6 @@ void tryLoadNext()
         }
 }
 
-void setCurrentSongToNext()
-{
-        if (currentSong != NULL)
-                lastPlayedId = currentSong->id;
-        currentSong = getNextSong();
-}
-
 void prepareNextSong()
 {
         if (!skipOutOfOrder && !isRepeatEnabled())
@@ -777,7 +886,7 @@ void prepareNextSong()
         }
         else
         {
-                notifySongSwitch();
+                determineSongAndNotify();
         }
 
         clock_gettime(CLOCK_MONOTONIC, &start_time);
@@ -799,27 +908,27 @@ gboolean mainloop_callback(gpointer data)
         (void)data;
 
         calcElapsedTime();
-        
-        handleInput();        
+
+        handleInput();
 
         updateCounter++;
-        
+
         // Update every other time or if searching (search needs to update often to detect keypresses)
-        if (updateCounter % 2 == 0 || appState.currentView == SEARCH_VIEW) 
+        if (updateCounter % 2 == 0 || appState.currentView == SEARCH_VIEW)
         {
                 // Process GDBus events in the global_main_context
                 while (g_main_context_pending(global_main_context))
                 {
                         g_main_context_iteration(global_main_context, FALSE);
                 }
-                
+
                 updatePlayer();
 
                 if (playlist.head != NULL)
                 {
                         if (loadingAudioData == false && (skipFromStopped || !loadedNextSong || nextSongNeedsRebuilding) && !audioData.endOfListReached)
                         {
-                                handleSkipFromStopped();
+                                // handleSkipFromStopped();
                                 loadAudioData();
                         }
 
@@ -835,6 +944,10 @@ gboolean mainloop_callback(gpointer data)
                                         switchAudioImplementation();
                         }
                 }
+                else
+                {
+                        setEOFNotReached();
+                }
 
                 if (doQuit)
                 {
@@ -845,7 +958,7 @@ gboolean mainloop_callback(gpointer data)
         return TRUE;
 }
 
-static gboolean on_sigint(gpointer user_data)
+static gboolean quitOnSignal(gpointer user_data)
 {
         doQuit = true;
         GMainLoop *loop = (GMainLoop *)user_data;
@@ -853,11 +966,17 @@ static gboolean on_sigint(gpointer user_data)
         return G_SOURCE_REMOVE; // Remove the signal source
 }
 
-void play(Node *song)
+void initFirstPlay(Node *song)
 {
         updateLastInputTime();
         updateLastSongSwitchTime();
 
+        userData.currentSongData = NULL;
+        userData.songdataA = NULL;
+        userData.songdataB = NULL;
+        userData.songdataADeleted = true;
+        userData.songdataBDeleted = true;
+
         int res = 0;
 
         if (song != NULL)
@@ -870,6 +989,10 @@ void play(Node *song)
                 {
                         res = createAudioDevice(&userData);
                 }
+                if (res >= 0)
+                {
+                        resumePlayback();
+                }
 
                 if (res < 0)
                         setEndOfListReached();
@@ -889,7 +1012,8 @@ void play(Node *song)
 
         main_loop = g_main_loop_new(NULL, FALSE);
 
-        g_unix_signal_add(SIGINT, on_sigint, main_loop);
+        g_unix_signal_add(SIGINT, quitOnSignal, main_loop);
+        g_unix_signal_add(SIGHUP, quitOnSignal, main_loop);
 
         if (song != NULL)
                 emitStartPlayingMpris();
@@ -908,16 +1032,18 @@ void cleanupOnExit()
         resetVorbisDecoders();
         resetOpusDecoders();
         resetM4aDecoders();
-        cleanupPlaybackDevice();
-        cleanupAudioContext();
+        if (isContextInitialized)
+        {
+                cleanupPlaybackDevice();
+                cleanupAudioContext();
+        }
         emitPlaybackStoppedMpris();
-        resetConsole();
+
+        bool noMusicFound = false;
 
         if (library == NULL || library->children == NULL)
         {
-                printf("No Music found.\n");
-                printf("Please make sure the path is set correctly. \n");
-                printf("To set it type: kew path \"/path/to/Music\". \n");
+                noMusicFound = true;
         }
 
         if (!userData.songdataADeleted)
@@ -947,12 +1073,28 @@ void cleanupOnExit()
         free(specialPlaylist);
         free(originalPlaylist);
         setDefaultTextColor();
-        showCursor();
         pthread_mutex_destroy(&(loadingdata.mutex));
         pthread_mutex_destroy(&(playlist.mutex));
         pthread_mutex_destroy(&(switchMutex));
         pthread_mutex_unlock(&dataSourceMutex);
         pthread_mutex_destroy(&(dataSourceMutex));
+#ifdef USE_LIBNOTIFY      
+        notify_uninit();
+#endif
+        resetConsole();
+        showCursor();
+        fflush(stdout);
+
+        if (noMusicFound)
+        {
+                printf("No Music found.\n");
+                printf("Please make sure the path is set correctly. \n");
+                printf("To set it type: kew path \"/path/to/Music\". \n");
+        }
+        else if (noPlaylist)
+        {
+                printf("Music not found.\n");
+        }
 
 #ifdef DEBUG
         fclose(logFile);
@@ -980,7 +1122,7 @@ void run()
         initMpris();
 
         currentSong = playlist.head;
-        play(currentSong);
+        initFirstPlay(currentSong);
         clearScreen();
         fflush(stdout);
 }
@@ -1008,10 +1150,13 @@ void init()
         pthread_mutex_init(&switchMutex, NULL);
         pthread_mutex_init(&(loadingdata.mutex), NULL);
         pthread_mutex_init(&(playlist.mutex), NULL);
-        nerdFontsEnabled = hasNerdFonts();
+        nerdFontsEnabled = true;
         createLibrary(&settings);
         setlocale(LC_ALL, "");
         fflush(stdout);
+#ifdef USE_LIBNOTIFY
+        notify_init("kew");
+#endif
 
 #ifdef DEBUG
         g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
@@ -1057,7 +1202,7 @@ void playAll()
         if (playlist.count == 0)
         {
                 exit(0);
-        }       
+        }
         shufflePlaylist(&playlist);
         run();
 }
@@ -1069,11 +1214,10 @@ void playAllAlbums()
         if (playlist.count == 0)
         {
                 exit(0);
-        }      
-        run();    
+        }
+        run();
 }
 
-
 void removeArgElement(char *argv[], int index, int *argc)
 {
         if (index < 0 || index >= *argc)
@@ -1150,8 +1294,165 @@ void handleOptions(int *argc, char *argv[])
                 removeArgElement(argv, idx, argc);
 }
 
+#define PIDFILE_TEMPLATE "/tmp/kew_%d.pid" // Template for user-specific PID file
+
+int isProcessRunning(pid_t pid)
+{
+        char proc_path[64];
+        snprintf(proc_path, sizeof(proc_path), "/proc/%d", pid);
+        struct stat statbuf;
+        return (stat(proc_path, &statbuf) == 0);
+}
+
+// Ensures only a single instance of kew can run at a time for the current user.
+void exitIfAlreadyRunning()
+{
+        char pidfile_path[256];
+        snprintf(pidfile_path, sizeof(pidfile_path), PIDFILE_TEMPLATE, getuid());
+
+        FILE *pidfile;
+        pid_t pid;
+
+        pidfile = fopen(pidfile_path, "r");
+        if (pidfile != NULL)
+        {
+                if (fscanf(pidfile, "%d", &pid) == 1)
+                {
+                        fclose(pidfile);
+                        if (isProcessRunning(pid))
+                        {
+                                fprintf(stderr, "An instance of kew is already running. Pid: %d.\n", pid);
+                                exit(EXIT_FAILURE);
+                        }
+                        else
+                        {
+                                unlink(pidfile_path);
+                        }
+                }
+                else
+                {
+                        fclose(pidfile);
+                        unlink(pidfile_path);
+                }
+        }
+
+        // Create a new PID file
+        pidfile = fopen(pidfile_path, "w");
+        if (pidfile == NULL)
+        {
+                perror("Unable to create PID file");
+                exit(EXIT_FAILURE);
+        }
+        fprintf(pidfile, "%d\n", getpid());
+        fclose(pidfile);
+}
+
+int directoryExists(const char *path)
+{
+    struct stat info;
+    if (stat(path, &info) != 0)
+    {
+        return 0;
+    }
+    else if (S_ISDIR(info.st_mode))
+    {
+        return 1;
+    }
+
+    return 0;
+}
+
+void setMusicPath()
+{
+        char *user = getenv("USER");
+
+        // Fallback if USER is not set
+        if (!user)
+        {
+                user = getlogin();
+
+                if (!user)
+                {
+                        struct passwd *pw = getpwuid(getuid());
+                        if (pw)
+                        {
+                                user = pw->pw_name;
+                        }
+                        else
+                        {
+                                printf("Error: Could not retrieve user information.\n");
+                                printf("Please set a path to your music library. \n");
+                                printf("To set it type: kew path \"/path/to/Music\". \n");
+                                exit(0);
+                        }
+                }
+        }
+
+        // Music folder names in different languages
+        const char *musicFolderNames[] = {
+            "Music", "Música", "Musique", "Musik", "Musica", "Muziek", "Музыка",
+            "音乐", "音楽", "음악", "موسيقى", "संगीत", "Müzik", "Musikk", "Μουσική",
+            "Muzyka", "Hudba", "Musiikki", "Zene", "Muzică", "เพลง", "מוזיקה"};
+
+        char path[MAXPATHLEN];
+        int found = 0;
+        char choice = ' ';
+        int result = -1;
+
+        for (size_t i = 0; i < sizeof(musicFolderNames) / sizeof(musicFolderNames[0]); i++)
+        {
+                snprintf(path, sizeof(path), "/home/%s/%s", user, musicFolderNames[i]);
+
+                if (directoryExists(path))
+                {
+                        found = 1;
+                        printf("Do you want to use %s as your music library folder?\n", path);
+                        printf("y = Yes\nn = Enter a path\n");
+
+                        result = scanf(" %c", &choice);
+
+                        if (choice == 'y' || choice == 'Y')
+                        {
+
+                                strncpy(settings.path, path, sizeof(settings.path));
+                                return;
+                        }
+                        else if (choice == 'n' || choice == 'N')
+                        {
+                                break; // Enter a custom path
+                        }
+                        else
+                        {
+                                printf("Invalid choice. Please try again.\n");
+                                i--;
+                        }
+                }
+        }
+
+        if (!found || (found && (choice == 'n' || choice == 'N')))
+        {
+                printf("Please enter the path to your music library (/path/to/Music):\n");
+                result = scanf("%s", path);
+
+                if (directoryExists(path))
+                {
+                        strncpy(settings.path, path, sizeof(settings.path));
+                }
+                else
+                {
+                        printf("The entered path does not exist.\n");
+                        exit(1);
+                }
+        }
+
+        if (result == -1)
+                exit(1);
+}
+
 int main(int argc, char *argv[])
 {
+        exitIfAlreadyRunning();
+
         if ((argc == 2 && ((strcmp(argv[1], "--help") == 0) || (strcmp(argv[1], "-h") == 0) || (strcmp(argv[1], "-?") == 0))))
         {
                 showHelp();
@@ -1163,7 +1464,6 @@ int main(int argc, char *argv[])
                 exit(0);
         }
 
-        moveConfigFiles(); // move from old location to new
         getConfig(&settings);
 
         if (argc == 3 && (strcmp(argv[1], "path") == 0))
@@ -1175,9 +1475,7 @@ int main(int argc, char *argv[])
 
         if (settings.path[0] == '\0')
         {
-                printf("Please make sure the path is set correctly. \n");
-                printf("To set it type: kew path \"/path/to/Music\". \n");
-                exit(0);
+                setMusicPath();
         }
 
         atexit(cleanupOnExit);
@@ -1206,7 +1504,10 @@ int main(int argc, char *argv[])
                 init();
                 makePlaylist(argc, argv, exactSearch, settings.path);
                 if (playlist.count == 0)
+                {
+                        noPlaylist = true;
                         exit(0);
+                }
                 run();
         }
 
