@@ -255,7 +255,8 @@ static void handle_stop(GDBusConnection *connection, const gchar *sender,
         (void)parameters;
         (void)user_data;
 
-        quit();
+        if (!isStopped())
+                stop();
         g_dbus_method_invocation_return_value(invocation, NULL);
 }
 
@@ -276,34 +277,78 @@ static void handle_play(GDBusConnection *connection, const gchar *sender,
         playbackPlay(&totalPauseSeconds, &pauseSeconds);
 }
 
-static void handle_seek(GDBusConnection *connection, const gchar *sender,
-                        const gchar *object_path, const gchar *interface_name,
-                        const gchar *method_name, GVariant *parameters,
-                        GDBusMethodInvocation *invocation, gpointer user_data)
+static void handle_seek(GDBusConnection *connection,
+                        const gchar *sender,
+                        const gchar *object_path,
+                        const gchar *interface_name,
+                        const gchar *method_name,
+                        GVariant *parameters,
+                        GDBusMethodInvocation *invocation,
+                        gpointer user_data)
 {
         (void)connection;
         (void)sender;
         (void)object_path;
         (void)interface_name;
         (void)method_name;
-        (void)parameters;
-        (void)invocation;        
-        (void)user_data;        
+        (void)user_data;
+
+        gint64 offset;
+        g_variant_get(parameters, "(x)", &offset);
+
+        gboolean success = seekPosition(offset);
+
+        if (success)
+        {
+                g_dbus_method_invocation_return_value(invocation, NULL);
+        }
+        else
+        {
+                g_dbus_method_invocation_return_error(invocation,
+                                                      G_DBUS_ERROR,
+                                                      G_DBUS_ERROR_FAILED,
+                                                      "Failed to seek to position");
+        }
 }
 
-static void handle_set_position(GDBusConnection *connection, const gchar *sender,
-                                const gchar *object_path, const gchar *interface_name,
-                                const gchar *method_name, GVariant *parameters,
-                                GDBusMethodInvocation *invocation, gpointer user_data)
+static void handle_set_position(GDBusConnection *connection,
+                                const gchar *sender,
+                                const gchar *object_path,
+                                const gchar *interface_name,
+                                const gchar *method_name,
+                                GVariant *parameters,
+                                GDBusMethodInvocation *invocation,
+                                gpointer user_data)
 {
         (void)connection;
         (void)sender;
         (void)object_path;
         (void)interface_name;
         (void)method_name;
-        (void)parameters;
-        (void)invocation;
-        (void)user_data;   
+        (void)user_data;
+
+        const gchar *track_id;
+        gint64 new_position;
+
+        // - "o" is an object path (or track identifier)
+        // - "x" is a 64-bit integer representing the position
+        g_variant_get(parameters, "(&ox)", &track_id, &new_position);
+
+        gboolean success = setPosition(new_position);
+
+        if (success)
+        {
+                // If setting the position was successful, return success with no additional value
+                g_dbus_method_invocation_return_value(invocation, NULL);
+        }
+        else
+        {
+                // If setting the position failed, return an error
+                g_dbus_method_invocation_return_error(invocation,
+                                                      G_DBUS_ERROR,
+                                                      G_DBUS_ERROR_FAILED,
+                                                      "Failed to set position for track %s", track_id);
+        }
 }
 
 static void handle_method_call(GDBusConnection *connection, const gchar *sender,
@@ -369,13 +414,11 @@ static void handle_method_call(GDBusConnection *connection, const gchar *sender,
         }
 }
 
-
 static void on_bus_name_acquired(GDBusConnection *connection, const gchar *name, gpointer user_data)
 {
         (void)connection;
         (void)name;
         (void)user_data;
-
 }
 
 static void on_bus_name_lost(GDBusConnection *connection, const gchar *name, gpointer user_data)
@@ -404,7 +447,7 @@ static gboolean get_playback_status(GDBusConnection *connection, const gchar *se
         {
                 status = "Paused";
         }
-        else if (currentSong == NULL)
+        else if (currentSong == NULL || isStopped())
         {
                 status = "Stopped";
         }
@@ -544,10 +587,13 @@ static gboolean get_volume(GDBusConnection *connection, const gchar *sender,
         (void)interface_name;
         (void)property_name;
         (void)error;
-        (void)user_data;        
+        (void)user_data;
 
         Volume = (gdouble)getCurrentVolume();
 
+        if (Volume >= 1)
+                Volume = Volume / 100;
+
         *value = g_variant_new_double(Volume);
         return TRUE;
 }
@@ -637,7 +683,7 @@ static gboolean get_can_go_previous(GDBusConnection *connection, const gchar *se
         (void)interface_name;
         (void)property_name;
         (void)error;
-        (void)user_data;        
+        (void)user_data;
 
         CanGoPrevious = (currentSong == NULL || currentSong->prev != NULL) ? TRUE : FALSE;
 
@@ -658,6 +704,11 @@ static gboolean get_can_play(GDBusConnection *connection, const gchar *sender,
         (void)error;
         (void)user_data;
 
+        if (currentSong == NULL)
+                CanPlay = FALSE;
+        else
+                CanPlay = TRUE;
+
         *value = g_variant_new_boolean(CanPlay);
         return TRUE;
 }
@@ -675,6 +726,11 @@ static gboolean get_can_pause(GDBusConnection *connection, const gchar *sender,
         (void)error;
         (void)user_data;
 
+        if (currentSong == NULL)
+                CanPause = FALSE;
+        else
+                CanPause = TRUE;
+
         *value = g_variant_new_boolean(CanPause);
         return TRUE;
 }
@@ -816,7 +872,7 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
         (void)object_path;
         (void)interface_name;
         (void)property_name;
-        (void)user_data;        
+        (void)user_data;
 
         if (g_strcmp0(interface_name, "org.mpris.MediaPlayer2.Player") == 0)
         {
@@ -829,6 +885,15 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
                 {
                         double new_volume;
                         g_variant_get(value, "d", &new_volume);
+
+                        if (new_volume > 1.0)
+                                new_volume = 1.0;
+
+                        if (new_volume < 0.0)
+                                new_volume = 0.0;
+
+                        new_volume *= 100;
+
                         setVolume((int)new_volume);
                         return TRUE;
                 }
@@ -844,12 +909,10 @@ static gboolean set_property_callback(GDBusConnection *connection, const gchar *
                 }
                 else if (g_strcmp0(property_name, "Position") == 0)
                 {
-                        // gint64 new_position;
-                        // g_variant_get(value, "x", &new_position);
-                        // set_position(new_position);
-                        // return TRUE;
-                        g_set_error(error, G_IO_ERROR, G_IO_ERROR_FAILED, "Setting Position property not supported");
-                        return FALSE;
+                        gint64 new_position;
+                        g_variant_get(value, "x", &new_position);
+
+                        return setPosition(new_position);
                 }
                 else
                 {
@@ -938,6 +1001,13 @@ void cleanupMpris()
                 g_main_context_unref(global_main_context);
                 global_main_context = NULL;
         }
+
+#ifdef USE_LIBNOTIFY
+        if (previous_notification != NULL) {
+                g_object_unref(previous_notification);
+                previous_notification = NULL;
+        }        
+#endif
 }
 
 void initMpris()
@@ -958,13 +1028,10 @@ void initMpris()
         }
 
         const char *app_name = "org.mpris.MediaPlayer2.kew";
-       
-        char unique_name[256];
-        snprintf(unique_name, sizeof(unique_name), "%s%d", app_name, getpid());
 
         GError *error = NULL;
         bus_name_id = g_bus_own_name_on_connection(connection,
-                                                   unique_name,
+                                                   app_name,
                                                    G_BUS_NAME_OWNER_FLAGS_NONE,
                                                    on_bus_name_acquired,
                                                    on_bus_name_lost,
@@ -973,7 +1040,7 @@ void initMpris()
 
         if (bus_name_id == 0)
         {
-                printf("Failed to own D-Bus name: %s\n", unique_name);
+                printf("Failed to own D-Bus name: %s\n", app_name);
                 exit(0);
         }
 
@@ -1026,13 +1093,100 @@ void emitStartPlayingMpris()
                                       NULL);
 }
 
+gchar *sanitizeTitle(const gchar *title)
+{
+    gchar *sanitized = g_strdup(title);
+
+    // Replace underscores with hyphens, otherwise some widgets have a problem
+    g_strdelimit(sanitized, "_", '-');
+
+    // duplicate string otherwise widgets have a problem with certain strings for some reason
+    gchar *sanitized_dup = g_strdup_printf("%s", sanitized);
+
+    g_free(sanitized);
+
+    return sanitized_dup;
+}
+
+static guint64 last_emit_time = 0;
+
+void emit_properties_changed(GDBusConnection *connection,
+                             const gchar *property_name,
+                             GVariant *new_value)
+{
+        GVariantBuilder changed_properties_builder;
+
+        if (connection == NULL || property_name == NULL || new_value == NULL)
+                return;
+
+        // Initialize the builder for changed properties
+        g_variant_builder_init(&changed_properties_builder, G_VARIANT_TYPE("a{sv}"));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", property_name, new_value);
+
+        GError *error = NULL;
+        gboolean result = g_dbus_connection_emit_signal(connection, NULL, "/org/mpris/MediaPlayer2", "org.freedesktop.DBus.Properties", "PropertiesChanged",
+                                                        g_variant_new("(sa{sv}as)", "org.mpris.MediaPlayer2.Player", &changed_properties_builder, NULL), &error);
+
+        if (!result)
+        {
+                g_critical("Failed to emit PropertiesChanged signal: %s", error->message);
+                g_error_free(error);
+        }
+        else
+        {
+                g_debug("PropertiesChanged signal emitted successfully.");
+        }
+
+        g_variant_builder_clear(&changed_properties_builder);
+}
+
+void emitVolumeChanged()
+{
+        gdouble newVolume = (gdouble)getCurrentVolume() / 100;
+
+        if (newVolume > 1.0)
+                return;
+
+        // Emit the PropertiesChanged signal for the volume property
+        GVariant *volume_variant = g_variant_new_double(newVolume);
+        emit_properties_changed(connection, "Volume", volume_variant);
+}
+
+void emitShuffleChanged()
+{
+        gboolean shuffleEnabled = isShuffleEnabled();
+
+        // Emit the PropertiesChanged signal for the volume property
+        GVariant *volume_variant = g_variant_new_boolean(shuffleEnabled);
+        emit_properties_changed(connection, "Shuffle", volume_variant);
+}
+
 void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *album, const gchar *coverArtPath, const gchar *trackId, Node *currentSong, gint64 length)
 {
-        gchar *coverArtUrl = g_strdup_printf("file://%s", coverArtPath);
+        guint64 current_time = g_get_monotonic_time();
+        if (current_time - last_emit_time < 500000) // 0.5 seconds
+        {
+                g_debug("Debounced signal emission.");
+                return;
+        }
 
+        last_emit_time = current_time;
+
+        if (!title || !album || !trackId)
+        {
+                g_warning("Invalid metadata: title, album, or trackId is NULL.");
+                return;
+        }
+
+        gchar *coverArtUrl = NULL;
+
+        gchar *sanitizedTitle = sanitizeTitle(title);
+
+        g_debug("Starting to build metadata.");
         GVariantBuilder metadata_builder;
         g_variant_builder_init(&metadata_builder, G_VARIANT_TYPE_DICTIONARY);
-        g_variant_builder_add(&metadata_builder, "{sv}", "xesam:title", g_variant_new_string(title));
+        g_variant_builder_add(&metadata_builder, "{sv}", "xesam:title", g_variant_new_string(sanitizedTitle));
+        g_free(sanitizedTitle);
 
         const gchar *artistList[2];
         if (artist)
@@ -1045,24 +1199,60 @@ void emitMetadataChanged(const gchar *title, const gchar *artist, const gchar *a
                 artistList[0] = "";
                 artistList[1] = NULL;
         }
-
         g_variant_builder_add(&metadata_builder, "{sv}", "xesam:artist", g_variant_new_strv(artistList, -1));
         g_variant_builder_add(&metadata_builder, "{sv}", "xesam:album", g_variant_new_string(album));
-        g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(coverArtUrl));
+
+        if (coverArtPath && *coverArtPath != '\0')
+        {
+                coverArtUrl = g_strdup_printf("file://%s", coverArtPath);
+                g_variant_builder_add(&metadata_builder, "{sv}", "mpris:artUrl", g_variant_new_string(coverArtUrl));
+                g_debug("Cover art URL added: %s", coverArtUrl);
+                g_free(coverArtUrl);
+        }
+
         g_variant_builder_add(&metadata_builder, "{sv}", "mpris:trackid", g_variant_new_object_path(trackId));
         g_variant_builder_add(&metadata_builder, "{sv}", "mpris:length", g_variant_new_int64(length));
 
+        GVariant *metadata_variant = g_variant_builder_end(&metadata_builder);
+
+        if (!metadata_variant)
+        {
+                g_warning("Failed to end metadata GVariantBuilder.");
+                return;
+        }
+
+        g_debug("Metadata built successfully.");
+
         GVariantBuilder changed_properties_builder;
         g_variant_builder_init(&changed_properties_builder, G_VARIANT_TYPE("a{sv}"));
-        g_variant_builder_add(&changed_properties_builder, "{sv}", "Metadata", g_variant_builder_end(&metadata_builder));
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "Metadata", metadata_variant);
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoPrevious", g_variant_new_boolean((currentSong != NULL && currentSong->prev != NULL)));
         g_variant_builder_add(&changed_properties_builder, "{sv}", "CanGoNext", g_variant_new_boolean((currentSong != NULL && currentSong->next != NULL)));
 
-        g_dbus_connection_emit_signal(connection, NULL, "/org/mpris/MediaPlayer2", "org.freedesktop.DBus.Properties", "PropertiesChanged",
-                                      g_variant_new("(sa{sv}as)", "org.mpris.MediaPlayer2.Player", &changed_properties_builder, NULL), NULL);
+        CanSeek = true;
+        if (currentSong != NULL && endsWith(currentSong->song.filePath, "ogg"))
+        {
+                CanSeek = false;
+        }
+
+        g_variant_builder_add(&changed_properties_builder, "{sv}", "CanSeek", g_variant_new_boolean(CanSeek));
+
+        g_debug("PropertiesChanged signal is ready to be emitted.");
+
+        GError *error = NULL;
+        gboolean result = g_dbus_connection_emit_signal(connection, NULL, "/org/mpris/MediaPlayer2", "org.freedesktop.DBus.Properties", "PropertiesChanged",
+                                                        g_variant_new("(sa{sv}as)", "org.mpris.MediaPlayer2.Player", &changed_properties_builder, NULL), &error);
+
+        if (!result)
+        {
+                g_critical("Failed to emit PropertiesChanged signal: %s", error->message);
+                g_error_free(error);
+        }
+        else
+        {
+                g_debug("PropertiesChanged signal emitted successfully.");
+        }
 
-        g_variant_builder_clear(&metadata_builder);
         g_variant_builder_clear(&changed_properties_builder);
-
-        g_free(coverArtUrl);
+        g_variant_builder_clear(&metadata_builder);
 }
