@@ -22,7 +22,7 @@ typedef struct
 } PixelData;
 #endif
 
-const char VERSION[] = "2.6.0";
+const char VERSION[] = "2.8.2";
 const int ABSOLUTE_MIN_WIDTH = 68;
 bool visualizerEnabled = true;
 bool coverEnabled = true;
@@ -87,14 +87,7 @@ FileSystemEntry *library = NULL;
 
 bool hasNerdFonts()
 {
-        bool nerdFonts = true;
-        if (printf("\uf28b") < 0)
-        {
-                nerdFonts = false;
-        }
-        clearScreen();
-        fflush(stdout);
-        return nerdFonts;
+        return (printf("\uf28b") >= 0); // nerd fonts
 }
 
 int calcMetadataHeight()
@@ -500,7 +493,7 @@ void printLastRow()
                 return;
         setTextColorRGB(lastRowColor.r, lastRowColor.g, lastRowColor.b);
 
-        char text[100] = " [F2 Playlist|F3 Library|F4 Track|F5 Search|F6 Keys|Esc Quit]";
+        char text[100] = " [F2 Playlist|F3 Library|F4 Track|F5 Search|F6 Help|Esc Quit]";
 
         char nerdFontText[100] = "";
 
@@ -589,11 +582,23 @@ int showKeyBindings(SongData *songdata, AppSettings *settings)
         setDefaultTextColor();
 
         printBlankSpaces(indent);
-        printf(" - Use %s (or %s) and %s to adjust volume.\n", settings->volumeUp, settings->volumeUpAlt, settings->volumeDown);
+        printf(" - Switch tracks with ←, → or %s, %s keys.\n", settings->previousTrackAlt, settings->nextTrackAlt);
         printBlankSpaces(indent);
-        printf(" - Use ←, → or %s, %s keys to switch tracks.\n", settings->previousTrackAlt, settings->nextTrackAlt);
+        printf(" - Volume is adjusted with %s (or %s) and %s.\n", settings->volumeUp, settings->volumeUpAlt, settings->volumeDown);
         printBlankSpaces(indent);
-        printf(" - Use ↑, ↓  or %s, %s keys to scroll through playlist.\n", settings->scrollUpAlt, settings->scrollDownAlt);
+        printf(" - Press F2 for Playlist View:\n");        
+        printBlankSpaces(indent);
+        printf("     Use ↑, ↓  or %s, %s keys to scroll through the playlist.\n", settings->scrollUpAlt, settings->scrollDownAlt);
+        printBlankSpaces(indent);
+        printf("     Press Enter to play.\n");
+        printBlankSpaces(indent);
+        printf(" - Press F3 for Library View:\n");
+        printBlankSpaces(indent);
+        printf("     Use ↑, ↓  or %s, %s keys to scroll through the library.\n", settings->scrollUpAlt, settings->scrollDownAlt);        
+        printBlankSpaces(indent);
+        printf("     Press Enter to add/remove songs to/from the playlist.\n");
+        printBlankSpaces(indent);
+        printf(" - Press F4 for Track View.\n");        
         printBlankSpaces(indent);
         printf(" - Enter a number then Enter to switch song.\n");
         printBlankSpaces(indent);
@@ -605,8 +610,6 @@ int showKeyBindings(SongData *songdata, AppSettings *settings)
         printBlankSpaces(indent);
         printf(" - %s to show/hide the spectrum visualizer.\n", settings->toggleVisualizer);
         printBlankSpaces(indent);
-        printf(" - %s to show/hide album covers.\n", settings->toggleCovers);
-        printBlankSpaces(indent);
         printf(" - %s to toggle album covers drawn in ascii.\n", settings->toggleAscii);
         printBlankSpaces(indent);
         printf(" - %s to repeat the current song.\n", settings->toggleRepeat);
@@ -625,7 +628,7 @@ int showKeyBindings(SongData *songdata, AppSettings *settings)
         printf("\n");
         printLastRow();
 
-        numPrintedRows += 17;
+        numPrintedRows += 23;
 
         return numPrintedRows;
 }
@@ -671,6 +674,31 @@ void toggleShowLibrary()
         }
 }
 
+void tabNext()
+{
+        if (appState.currentView == PLAYLIST_VIEW)
+                appState.currentView = LIBRARY_VIEW;
+        else if (appState.currentView == LIBRARY_VIEW)
+        {
+                if (currentSong != NULL)
+                {
+                        appState.currentView = SONG_VIEW;
+                }
+                else
+                {
+                        appState.currentView = SEARCH_VIEW;
+                }
+        }
+        else if (appState.currentView == SONG_VIEW)
+                appState.currentView = SEARCH_VIEW;
+        else if (appState.currentView == SEARCH_VIEW)
+                appState.currentView = KEYBINDINGS_VIEW;
+        else if (appState.currentView == KEYBINDINGS_VIEW)
+                appState.currentView = PLAYLIST_VIEW;
+                
+        refresh = true;
+}
+
 void showTrack()
 {
         refresh = true;
@@ -978,10 +1006,20 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
 {
         char dirName[maxNameWidth + 1];
         char filename[maxNameWidth + 1];
+        bool foundChosen = false;
+        int foundCurrent = 0;
+
+        if (currentSong != NULL && (strcmp(currentSong->song.filePath, root->fullPath) == 0))
+        {
+                foundCurrent = 1;
+        }
+
+        if (startLibIter < 0)
+                startLibIter = 0;
 
         if (libIter >= startLibIter + maxListSize)
         {
-                return 0;
+                return false;
         }
 
         FileSystemEntry *tmp = root->parent == NULL ? NULL : root->parent->children;
@@ -1003,12 +1041,8 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
         if (allowChooseSongs)
         {
                 if (chosenLibRow >= libIter + libSongIter && libSongIter != 0)
-                {
-                        if (chosenLibRow >= numDirectoryTreeEntries + numTopLevelSongs + numAudioChildren)
-                        {
-                                startLibIter = numDirectoryTreeEntries + numTopLevelSongs + numAudioChildren - maxListSize;
-                                chosenLibRow = numDirectoryTreeEntries + numTopLevelSongs + numAudioChildren - 1;
-                        }
+                {                               
+                        startLibIter = chosenLibRow - round(maxListSize / 2);
                 }
         }
         else
@@ -1024,7 +1058,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                 startLibIter = chosenLibRow = libIter = 0;
 
         if (root == NULL)
-                return 0;
+                return false;
 
         if (root->isDirectory ||
             (!root->isDirectory && depth == 1) ||
@@ -1060,6 +1094,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                                         setTextColor(enqueuedColor);
                                                 else
                                                         setColor();
+
                                                 printf("\x1b[7m * ");
                                         }
                                         else
@@ -1078,19 +1113,25 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                                 chosenDir = NULL;
                                                 refresh = true;
                                         }
+
+                                        foundChosen = true;                               
                                 }
                                 else
                                 {
                                         if (root->isEnqueued)
                                         {
+
                                                 if (useProfileColors)
-                                                        setTextColor(enqueuedColor);
+                                                        printf("\033[%d;3%dm", foundCurrent, enqueuedColor);
                                                 else
-                                                        setColor();
+                                                        setColorAndWeight(foundCurrent);
+
                                                 printf(" * ");
                                         }
                                         else
+                                        {
                                                 printf("   ");
+                                        }
                                 }
 
                                 if (root->isDirectory)
@@ -1122,13 +1163,14 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                 FileSystemEntry *child = root->children;
                 while (child != NULL)
                 {
-                        if (displayTree(child, depth + 1, maxListSize, maxNameWidth) == -1)
-                                return -1;
+                        if (displayTree(child, depth + 1, maxListSize, maxNameWidth))
+                                foundChosen = true;
+
                         child = child->next;
                 }
         }
 
-        return 0;
+        return foundChosen;
 }
 
 char *getLibraryFilePath()
@@ -1185,7 +1227,13 @@ void showLibrary(SongData *songData)
                 tmp = tmp->next;
         }
 
-        displayTree(library, 0, maxLibListSize, maxNameWidth);
+        bool foundChosen = displayTree(library, 0, maxLibListSize, maxNameWidth);
+
+        if (!foundChosen)
+        {
+                chosenLibRow--;
+                refresh = true;
+        }
 
         printf("\n");
 
@@ -1249,7 +1297,7 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         if (appState.currentView == KEYBINDINGS_VIEW && refresh)
         {
                 clearScreen();
-                showKeyBindings(songdata, settings);
+                showKeyBindings(songdata, settings); 
                 saveCursorPosition();
                 refresh = false;
         }
