@@ -28,6 +28,7 @@ bool enqueuedNeedsUpdate = false;
 bool skipFromStopped = false;
 bool playingMainPlaylist = false;
 bool usingSongDataA = true;
+bool doNotifyMPRISSwitched = false;
 
 LoadingThreadData loadingdata;
 
@@ -36,6 +37,7 @@ bool waitingForPlaylist = false;
 bool waitingForNext = false;
 Node *nextSong = NULL;
 Node *tryNextSong = NULL;
+Node *songToStartFrom = NULL;
 Node *prevSong = NULL;
 int lastPlayedId = -1;
 
@@ -48,7 +50,7 @@ volatile bool clearingErrors = false;
 volatile bool songLoading = false;
 GDBusConnection *connection = NULL;
 
-void updatePlaylist()
+void reshufflePlaylist()
 {
         playlistNeedsUpdate = false;
 
@@ -72,7 +74,7 @@ void rebuildAndUpdatePlaylist()
 
         if (playlistNeedsUpdate)
         {
-                updatePlaylist();
+                reshufflePlaylist();
         }
 
         pthread_mutex_unlock(&(playlist.mutex));
@@ -178,6 +180,99 @@ void playbackPause(struct timespec *pause_time)
         pausePlayback();
 }
 
+void skipToSong(int id, bool startPlaying)
+{
+        if (songLoading || !loadedNextSong || skipping || clearingErrors)
+                if (!forceSkip)
+                        return;
+
+        Node *found = NULL;
+        findNodeInList(&playlist, id, &found);
+
+        if (found != NULL)
+                currentSong = found;
+        else
+        {
+                return;
+        }
+
+        skipping = true;
+        skipOutOfOrder = true;
+        loadedNextSong = false;
+        songLoading = true;
+        forceSkip = false;
+
+        if (startPlaying)
+        {
+                playbackPlay(&totalPauseSeconds, &pauseSeconds);
+        }
+
+        // cancel starting from top
+        if (waitingForPlaylist || audioData.restart)
+        {
+                waitingForPlaylist = false;
+                audioData.restart = false;
+
+                if (isShuffleEnabled())
+                        reshufflePlaylist();
+        }
+
+        loadingdata.loadA = !usingSongDataA;
+
+        loadingdata.loadingFirstDecoder = true;
+        loadSong(currentSong, &loadingdata);
+
+        int maxNumTries = 50;
+        int numtries = 0;
+
+        while (!loadedNextSong && !loadingFailed && numtries < maxNumTries)
+        {
+                c_sleep(100);
+                numtries++;
+        }
+
+        if (songHasErrors)
+        {
+                songHasErrors = false;
+                forceSkip = true;
+                if (currentSong->next != NULL)
+                        skipToSong(currentSong->next->id, true);
+        }
+
+        updateLastSongSwitchTime();
+        skip();
+}
+
+void skipToBegginningOfSong()
+{
+        if (currentSong != NULL)
+        {
+                bool playSong = false;
+                skipToSong(currentSong->id, playSong);
+        }
+}
+
+void prepareIfSkippedSilent()
+{
+        if (hasSilentlySwitched)
+        {
+                skipping = true;
+                hasSilentlySwitched = false;
+                updateLastSongSwitchTime();
+                setCurrentImplementationType(NONE);
+                setRepeatEnabled(false);
+                audioData.endOfListReached = false;
+
+                rebuildAndUpdatePlaylist();
+
+                switchAudioImplementation();
+
+                usingSongDataA = !usingSongDataA;
+
+                skipping = false;
+        }
+}
+
 void playbackPlay(double *totalPauseSeconds, double *pauseSeconds)
 {
         if (isPaused())
@@ -185,7 +280,23 @@ void playbackPlay(double *totalPauseSeconds, double *pauseSeconds)
                 *totalPauseSeconds += *pauseSeconds;
                 emitStringPropertyChanged("PlaybackStatus", "Playing");
         }
+        else if (isStopped())
+        {
+                emitStringPropertyChanged("PlaybackStatus", "Playing");
+        }
+
+        if (isStopped() && !hasSilentlySwitched)
+        {
+                skipToBegginningOfSong();
+        }
+
         resumePlayback();
+
+        if (hasSilentlySwitched)
+        {
+                *totalPauseSeconds = 0;
+                prepareIfSkippedSilent();
+        }
 }
 
 void togglePause(double *totalPauseSeconds, double *pauseSeconds, struct timespec *pause_time)
@@ -199,7 +310,15 @@ void togglePause(double *totalPauseSeconds, double *pauseSeconds, struct timespe
         }
         else
         {
-                *totalPauseSeconds += *pauseSeconds;
+                if (hasSilentlySwitched && !skipping)
+                {
+                        *totalPauseSeconds = 0;
+                        prepareIfSkippedSilent();
+                }
+                else
+                {
+                        *totalPauseSeconds += *pauseSeconds;
+                }
                 emitStringPropertyChanged("PlaybackStatus", "Playing");
         }
 }
@@ -303,14 +422,6 @@ void toggleColors(AppSettings *settings)
         refresh = true;
 }
 
-void toggleCovers(AppSettings *settings)
-{
-        coverEnabled = !coverEnabled;
-        c_strcpy(settings->coverEnabled, sizeof(settings->coverEnabled), coverEnabled ? "1" : "0");
-        clearScreen();
-        refresh = true;
-}
-
 void toggleVisualizer(AppSettings *settings)
 {
         visualizerEnabled = !visualizerEnabled;
@@ -342,16 +453,24 @@ SongData *getCurrentSongData(void)
         if (isCurrentSongDeleted())
                 return NULL;
 
-        SongData *songData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
+        SongData *songData = NULL;
+
+        bool isDeleted = determineCurrentSongData(&songData);
+
+        if (isDeleted)
+                return NULL;
 
         if (!isValidSong(songData))
-                return NULL;                
+                return NULL;
 
         return songData;
 }
 
 void calcElapsedTime()
 {
+        if (isStopped())
+                return;
+
         clock_gettime(CLOCK_MONOTONIC, &current_time);
 
         double timeSinceLastUpdate = (double)(current_time.tv_sec - lastUpdateTime.tv_sec) +
@@ -380,7 +499,6 @@ void calcElapsedTime()
 
                 if (currentSong != NULL && timeSinceLastUpdate >= 1.0)
                 {
-                        updatePlaybackPosition(elapsedSeconds);
                         lastUpdateTime = current_time;
                 }
         }
@@ -420,6 +538,38 @@ void flushSeek()
         }
 }
 
+bool setPosition(gint64 newPosition)
+{
+        gint64 currentPositionMicroseconds = llround(elapsedSeconds * G_USEC_PER_SEC);
+
+        if (duration != 0.0)
+        {
+                gint64 step = newPosition - currentPositionMicroseconds;
+                step = step / G_USEC_PER_SEC;
+                seekAccumulatedSeconds += step;
+                return true;
+        }
+        else
+        {
+                return false;
+        }
+}
+
+bool seekPosition(gint64 offset)
+{
+        if (duration != 0.0)
+        {
+                gint64 step = offset;
+                step = step / G_USEC_PER_SEC;
+                seekAccumulatedSeconds += step;
+                return true;
+        }
+        else
+        {
+                return false;
+        }
+}
+
 void seekForward()
 {
         if (currentSong != NULL)
@@ -594,6 +744,51 @@ void enqueueSong(FileSystemEntry *child)
         child->parent->isEnqueued = 1;
 }
 
+void silentSwitchToNext(bool loadSong)
+{
+        skipping = true;
+        nextSong = NULL;
+        setCurrentSongToNext();
+        activateSwitch(&audioData);
+
+        skipOutOfOrder = true;
+
+        usingSongDataA = (audioData.currentFileIndex == 0);
+
+        if (loadSong)
+        {
+                loadNextSong();
+                finishLoading();
+                loadedNextSong = true;
+                doNotifyMPRISSwitched = true;
+        }
+
+        resetTimeCount();
+        clock_gettime(CLOCK_MONOTONIC, &start_time);
+
+        refresh = true;
+
+        skipping = false;
+
+        hasSilentlySwitched = true;
+}
+
+void removeCurrentlyPlayingSong()
+{
+        stopPlayback();
+        emitStringPropertyChanged("PlaybackStatus", "Stopped");
+
+        clearCurrentTrack();
+
+        loadedNextSong = false;
+        audioData.restart = true;
+        audioData.endOfListReached = true;
+        lastPlayedId = currentSong->id;
+        songToStartFrom = getListNext(currentSong);
+        waitingForNext = true;
+        currentSong = NULL;
+}
+
 void dequeueSong(FileSystemEntry *child)
 {
         Node *node1 = findLastPathInPlaylist(child->fullPath, originalPlaylist);
@@ -602,7 +797,16 @@ void dequeueSong(FileSystemEntry *child)
                 return;
 
         if (currentSong != NULL && currentSong->id == node1->id)
-                return;
+        {
+                removeCurrentlyPlayingSong();
+        }
+        else
+        {
+                if (songToStartFrom != NULL)
+                {
+                        songToStartFrom = getListNext(node1);
+                }
+        }
 
         int id = node1->id;
 
@@ -714,11 +918,6 @@ bool hasDequeuedChildren(FileSystemEntry *parent)
         return isDequeued;
 }
 
-void dequeueAll()
-{
-        
-}
-
 void enqueueSongs(FileSystemEntry *entry)
 {
         FileSystemEntry *chosenDir = getChosenDir();
@@ -782,7 +981,7 @@ void enqueueSongs(FileSystemEntry *entry)
 
         if (nextSongNeedsRebuilding)
         {
-                updatePlaylist();
+                reshufflePlaylist();
         }
         updateLastPlaylistChangeTime();
 }
@@ -811,19 +1010,21 @@ void handleRemove()
 
         int currentId = (currentSong != NULL) ? currentSong->id : -1;
 
-        if (node != NULL && playlist.head != NULL && playlist.head->id == node->id && playlist.count == 1)
-        {
-                return;
-        }
-
         if (currentId == node->id)
         {
-                return;
+                removeCurrentlyPlayingSong();
+        }
+        else
+        {
+                if (songToStartFrom != NULL)
+                {
+                        songToStartFrom = getListNext(node);
+                }
         }
 
         pthread_mutex_lock(&(playlist.mutex));
 
-        if (node != NULL && song != NULL)
+        if (node != NULL && song != NULL && currentSong != NULL)
         {
                 if (strcmp(song->song.filePath, node->song.filePath) == 0 || (currentSong != NULL && currentSong->next != NULL && id == currentSong->next->id))
                         rebuild = true;
@@ -847,11 +1048,11 @@ void handleRemove()
 
         currentSong = findSelectedEntryById(&playlist, currentId);
 
-        if (rebuild)
+        if (rebuild && currentSong != NULL)
         {
                 node = NULL;
                 nextSong = NULL;
-                updatePlaylist();
+                reshufflePlaylist();
 
                 tryNextSong = currentSong->next;
                 nextSongNeedsRebuilding = false;
@@ -905,7 +1106,7 @@ int loadDecoder(SongData *songData, bool *songDataDeleted)
                                 result = prepareNextOpusDecoder(songData->filePath);
                         else if (endsWith(songData->filePath, "ogg"))
                                 result = prepareNextVorbisDecoder(songData->filePath);
-                        else if (endsWith(songData->filePath, "m4a") || endsWith(songData->filePath, "aac") || endsWith(songData->filePath, "mp4"))
+                        else if (endsWith(songData->filePath, "m4a") || endsWith(songData->filePath, "aac"))
                                 result = prepareNextM4aDecoder(songData->filePath);
                 }
         }
@@ -1060,36 +1261,106 @@ void rebuildNextSong(Node *song)
         songLoading = false;
 }
 
+void stop()
+{
+        stopPlayback();
+
+        if (isStopped())
+        {
+                emitStringPropertyChanged("PlaybackStatus", "Stopped");
+        }
+}
+
+void loadNextSong()
+{
+        songLoading = true;
+        nextSongNeedsRebuilding = false;
+        skipFromStopped = false;
+        loadingdata.loadA = !usingSongDataA;
+        tryNextSong = nextSong = getListNext(currentSong);
+        loadingdata.loadingFirstDecoder = false;
+        loadSong(nextSong, &loadingdata);
+}
+
+bool determineCurrentSongData(SongData **currentSongData)
+{
+        *currentSongData = (audioData.currentFileIndex == 0) ? userData.songdataA : userData.songdataB;
+        bool isDeleted = (audioData.currentFileIndex == 0) ? userData.songdataADeleted == true : userData.songdataBDeleted == true;
+
+        if (isDeleted)
+        {
+                *currentSongData = (audioData.currentFileIndex != 0) ? userData.songdataA : userData.songdataB;
+                isDeleted = (audioData.currentFileIndex != 0) ? userData.songdataADeleted == true : userData.songdataBDeleted == true;
+
+                if (!isDeleted)
+                {
+                        activateSwitch(&audioData);
+                        audioData.switchFiles = false;
+                }
+        }
+        return isDeleted;
+}
+
+void setCurrentSongToNext()
+{
+        if (currentSong != NULL)
+                lastPlayedId = currentSong->id;
+        currentSong = getNextSong();
+}
+
+void finishLoading()
+{
+        int maxNumTries = 20;
+        int numtries = 0;
+
+        while (!loadedNextSong && !loadingFailed && numtries < maxNumTries)
+        {
+                c_sleep(100);
+                numtries++;
+        }
+
+        loadedNextSong = true;
+}
+
+void resetTimeCount()
+{
+        elapsedSeconds = 0.0;
+        pauseSeconds = 0.0;
+        totalPauseSeconds = 0.0;
+}
+
 void skipToNextSong()
 {
+        // Stop if there is no song or no next song
         if (currentSong == NULL || currentSong->next == NULL)
         {
+                if (!isStopped() && !isPaused())
+                        stop();
                 return;
         }
 
         if (songLoading || nextSongNeedsRebuilding || skipping || clearingErrors)
                 return;
 
+        if (isStopped() || isPaused())
+        {
+                // FIXME: Emit MPRIS signal
+                silentSwitchToNext(true);
+                return;
+        }
+
         playbackPlay(&totalPauseSeconds, &pauseSeconds);
 
         skipping = true;
+        skipOutOfOrder = false;
 
         updateLastSongSwitchTime();
 
         skip();
 }
 
-void skipToPrevSong()
+void setCurrentSongToPrev()
 {
-        if ((currentSong == NULL || currentSong->prev == NULL) && !isShuffleEnabled())
-        {
-                return;
-        }
-
-        if (songLoading || skipping || clearingErrors)
-                if (!forceSkip)
-                        return;
-
         if (isShuffleEnabled() && currentSong != NULL && currentSong->prev == NULL)
         {
                 if (currentSong->prev == NULL && currentSong->next != NULL)
@@ -1099,6 +1370,58 @@ void skipToPrevSong()
         }
         else
                 currentSong = currentSong->prev;
+}
+
+void silentSwitchToPrev()
+{
+        skipping = true;
+
+        setCurrentSongToPrev();
+        activateSwitch(&audioData);
+
+        loadedNextSong = false;
+        songLoading = true;
+        forceSkip = false;
+
+        usingSongDataA = !usingSongDataA;
+
+        loadingdata.loadA = usingSongDataA;
+        loadingdata.loadingFirstDecoder = true;
+        loadSong(currentSong, &loadingdata);
+        doNotifyMPRISSwitched = true;
+        finishLoading();
+
+        resetTimeCount();
+        clock_gettime(CLOCK_MONOTONIC, &start_time);
+
+        refresh = true;
+        skipping = false;
+
+        skipOutOfOrder = true;
+        hasSilentlySwitched = true;
+}
+
+void skipToPrevSong()
+{
+        // Stop if there is no song or no previous song
+        if ((currentSong == NULL || currentSong->prev == NULL) && !isShuffleEnabled())
+        {
+                if (!isStopped() && !isPaused())
+                        stop();
+                return;
+        }
+
+        if (songLoading || skipping || clearingErrors)
+                if (!forceSkip)
+                        return;
+
+        if (isStopped() || isPaused())
+        {
+                silentSwitchToPrev();
+                return;
+        }
+
+        setCurrentSongToPrev();
 
         playbackPlay(&totalPauseSeconds, &pauseSeconds);
 
@@ -1131,54 +1454,6 @@ void skipToPrevSong()
         skip();
 }
 
-void skipToSong(int id)
-{
-        if (songLoading || !loadedNextSong || skipping || skipOutOfOrder || clearingErrors)
-                if (!forceSkip)
-                        return;
-
-        Node *found = NULL;
-        findNodeInList(&playlist, id, &found);
-
-        if (found != NULL)
-                currentSong = found;
-        else
-        {
-                return;
-        }
-
-        skipping = true;
-        skipOutOfOrder = true;
-        loadedNextSong = false;
-        songLoading = true;
-        forceSkip = false;
-
-        playbackPlay(&totalPauseSeconds, &pauseSeconds);
-
-        loadingdata.loadA = !usingSongDataA;
-        loadingdata.loadingFirstDecoder = true;
-        loadSong(currentSong, &loadingdata);
-        int maxNumTries = 50;
-        int numtries = 0;
-
-        while (!loadedNextSong && !loadingFailed && numtries < maxNumTries)
-        {
-                c_sleep(100);
-                numtries++;
-        }
-
-        if (songHasErrors)
-        {
-                songHasErrors = false;
-                forceSkip = true;
-                if (currentSong->next != NULL)
-                        skipToSong(currentSong->next->id);
-        }
-
-        updateLastSongSwitchTime();
-        skip();
-}
-
 void skipToNumberedSong(int songNumber)
 {
         if (songLoading || !loadedNextSong || skipping || clearingErrors)
@@ -1253,6 +1528,26 @@ void loadFirstSong(Node *song)
         }
 }
 
+void unloadSongA()
+{
+        if (userData.songdataADeleted == false)
+        {
+                userData.songdataADeleted = true;
+                unloadSongData(&loadingdata.songdataA);
+                userData.songdataA = NULL;
+        }
+}
+
+void unloadSongB()
+{
+        if (userData.songdataBDeleted == false)
+        {
+                userData.songdataBDeleted = true;
+                unloadSongData(&loadingdata.songdataB);
+                userData.songdataB = NULL;
+        }
+}
+
 void unloadPreviousSong()
 {
         pthread_mutex_lock(&(loadingdata.mutex));
@@ -1261,21 +1556,23 @@ void unloadPreviousSong()
             (skipping || (userData.currentSongData == NULL || userData.songdataADeleted == false ||
                           (loadingdata.songdataA != NULL && userData.songdataADeleted == false && userData.currentSongData->hasErrors == 0 && userData.currentSongData->trackId != NULL && strcmp(loadingdata.songdataA->trackId, userData.currentSongData->trackId) != 0))))
         {
-                userData.songdataADeleted = true;
-                unloadSongData(&loadingdata.songdataA);
-                usingSongDataA = false;
+                unloadSongA();
+
                 if (!audioData.endOfListReached)
                         loadedNextSong = false;
+
+                usingSongDataA = false;
         }
         else if (!usingSongDataA &&
                  (skipping || (userData.currentSongData == NULL || userData.songdataBDeleted == false ||
                                (loadingdata.songdataB != NULL && userData.songdataBDeleted == false && userData.currentSongData->hasErrors == 0 && userData.currentSongData->trackId != NULL && strcmp(loadingdata.songdataB->trackId, userData.currentSongData->trackId) != 0))))
         {
-                userData.songdataBDeleted = true;
-                unloadSongData(&loadingdata.songdataB);
-                usingSongDataA = true;
+                unloadSongB();
+
                 if (!audioData.endOfListReached)
                         loadedNextSong = false;
+
+                usingSongDataA = true;
         }
 
         pthread_mutex_unlock(&(loadingdata.mutex));
@@ -1319,6 +1616,8 @@ void *updateLibraryThread(void *arg)
 
         pthread_mutex_lock(&switchMutex);
 
+        copyIsEnqueued(library, temp);
+
         freeTree(library);
         library = temp;
         numDirectoryTreeEntries = tmpDirectoryTreeEntries;
@@ -1391,6 +1690,7 @@ void createLibrary(AppSettings *settings)
                 char *libFilepath = getLibraryFilePath();
                 library = reconstructTreeFromFile(libFilepath, settings->path, &numDirectoryTreeEntries);
                 free(libFilepath);
+                updateLibraryIfChangedDetected();                
         }
 
         if (library == NULL || library->children == NULL)
@@ -1418,3 +1718,72 @@ void createLibrary(AppSettings *settings)
                 exit(0);
         }
 }
+
+time_t getModificationTime(struct stat *path_stat) {
+
+    if (path_stat->st_mtime != 0) {
+        return path_stat->st_mtime;
+    } else {
+        return path_stat->st_mtim.tv_sec;  // Fallback to st_mtim.tv_sec if st_mtime is zero or invalid
+    }
+}
+
+void *updateIfTopLevelFoldersMtimesChangedThread(void *arg)
+{
+    char *path = (char *)arg;
+    struct stat path_stat;
+
+    if (stat(path, &path_stat) == -1) {
+        perror("stat");
+        pthread_exit(NULL);
+    }
+
+    if (getModificationTime(&path_stat) > lastTimeAppRan && lastTimeAppRan > 0) {        
+        updateLibrary(path);
+        pthread_exit(NULL);
+    }
+
+    DIR *dir = opendir(path);
+    if (!dir) {
+        perror("opendir");
+        pthread_exit(NULL);
+    }
+
+    struct dirent *entry;
+    while ((entry = readdir(dir)) != NULL) {
+
+        if (strcmp(entry->d_name, ".") == 0 || strcmp(entry->d_name, "..") == 0) {
+            continue;
+        }
+
+        char fullPath[1024];
+        snprintf(fullPath, sizeof(fullPath), "%s/%s", path, entry->d_name);
+
+        if (stat(fullPath, &path_stat) == -1) {
+            perror("stat");
+            continue;
+        }
+
+        if (S_ISDIR(path_stat.st_mode)) {
+
+            if (getModificationTime(&path_stat) > lastTimeAppRan && lastTimeAppRan > 0) {                
+                updateLibrary(path);
+                break;
+            }
+        }
+    }
+
+    closedir(dir);
+
+    pthread_exit(NULL);
+}
+
+// This only checks the library mtime and toplevel subfolders mtimes
+void updateLibraryIfChangedDetected()
+{
+        pthread_t tid;
+        if (pthread_create(&tid, NULL, updateIfTopLevelFoldersMtimesChangedThread, (void *)settings.path) != 0)
+        {
+                perror("pthread_create");
+        }
+}
