@@ -580,9 +580,9 @@ int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path)
         return 0;
 }
 
-// Function to generate the filename with the .m3u extension
+
 void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFilename, size_t size) {
-    // Find the last occurrence of '/' in the file path to get the base name
+
     const char *baseName = strrchr(filePath, '/');
     if (baseName == NULL) {
         baseName = filePath; // No '/' found, use the entire filename
@@ -590,7 +590,6 @@ void generateM3UFilename(const char *basePath, const char *filePath, char *m3uFi
         baseName++; // Skip the '/' character
     }
 
-    // Find the last occurrence of '.' in the base name
     const char *dot = strrchr(baseName, '.');
     if (dot == NULL) {
         // No '.' found, copy the base name and append ".m3u"
@@ -842,7 +841,7 @@ void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, in
 
 int isMusicFile(const char *filename)
 {
-    const char *extensions[] = {".m4a", ".aac",".mp4", ".mp3", ".ogg", ".flac",".wav", ".opus"};
+    const char *extensions[] = {".m4a", ".aac", ".mp3", ".ogg", ".flac",".wav", ".opus"};
 
     size_t numExtensions = sizeof(extensions) / sizeof(extensions[0]);
 
