@@ -68,6 +68,8 @@ void shufflePlaylistStartingFromSong(PlayList *playlist, Node *song);
 
 int makePlaylist(int argc, char *argv[], bool exactSearch, const char *path);
 
+void writeCurrentPlaylistToM3UFile(PlayList *playlist);
+
 void writeM3UFile(const char *filename, PlayList *playlist);
 
 void loadSpecialPlaylist(const char *directory);
@@ -87,3 +89,5 @@ Node *findLastPathInPlaylist(char *path, PlayList *playlist);
 int findNodeInList(PlayList *list, int id, Node **foundNode);
 
 void createPlayListFromFileSystemEntry(FileSystemEntry *root, PlayList *list, int playlistMax);
+
+void addShuffledAlbumsToPlayList(FileSystemEntry *root, PlayList *list, int playlistMax);
