@@ -14,6 +14,8 @@ settings.c
 
 const char SETTINGS_FILE[] = "kewrc";
 
+time_t lastTimeAppRan;
+
 void freeKeyValuePairs(KeyValuePair *pairs, int count)
 {
         for (int i = 0; i < count; i++)
@@ -33,11 +35,12 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         strncpy(settings.allowNotifications, "1", sizeof(settings.allowNotifications));
         strncpy(settings.coverAnsi, "0", sizeof(settings.coverAnsi));
         strncpy(settings.visualizerEnabled, "1", sizeof(settings.visualizerEnabled));
-        strncpy(settings.useProfileColors, "1", sizeof(settings.useProfileColors));
+        strncpy(settings.useProfileColors, "0", sizeof(settings.useProfileColors));
         strncpy(settings.hideLogo, "0", sizeof(settings.hideLogo));
         strncpy(settings.hideHelp, "0", sizeof(settings.hideHelp));
         strncpy(settings.cacheLibrary, "-1", sizeof(settings.cacheLibrary));
 
+        strncpy(settings.tabNext, "\t", sizeof(settings.tabNext));
         strncpy(settings.volumeUp, "+", sizeof(settings.volumeUp));
         strncpy(settings.volumeUpAlt, "=", sizeof(settings.volumeUpAlt));
         strncpy(settings.volumeDown, "-", sizeof(settings.volumeDown));
@@ -47,7 +50,6 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         strncpy(settings.scrollDownAlt, "j", sizeof(settings.scrollDownAlt));
         strncpy(settings.toggleColorsDerivedFrom, "i", sizeof(settings.toggleColorsDerivedFrom));
         strncpy(settings.toggleVisualizer, "v", sizeof(settings.toggleVisualizer));
-        strncpy(settings.toggleCovers, "c", sizeof(settings.toggleCovers));
         strncpy(settings.toggleAscii, "b", sizeof(settings.toggleAscii));
         strncpy(settings.toggleRepeat, "r", sizeof(settings.toggleRepeat));
         strncpy(settings.toggleShuffle, "s", sizeof(settings.toggleShuffle));
@@ -63,20 +65,26 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         strncpy(settings.hardNext, "[C", sizeof(settings.hardNext));
         strncpy(settings.hardScrollUp, "[A", sizeof(settings.hardScrollUp));
         strncpy(settings.hardScrollDown, "[B", sizeof(settings.hardScrollDown));
-        strncpy(settings.hardShowInfo, "OQ", sizeof(settings.hardShowInfo));
-        strncpy(settings.hardShowInfoAlt, "[[B", sizeof(settings.hardShowInfoAlt));
+        strncpy(settings.hardShowPlaylist, "OQ", sizeof(settings.hardShowPlaylist));
+        strncpy(settings.hardShowPlaylistAlt, "[[B", sizeof(settings.hardShowPlaylistAlt));
+        strncpy(settings.showPlaylistAlt, "", sizeof(settings.showPlaylistAlt));
         strncpy(settings.hardShowKeys, "[17~", sizeof(settings.hardShowKeys));
         strncpy(settings.hardShowKeysAlt, "[17~", sizeof(settings.hardShowKeysAlt));
+        strncpy(settings.showKeysAlt, "", sizeof(settings.showKeysAlt));
         strncpy(settings.hardShowTrack, "OS", sizeof(settings.hardShowTrack));
         strncpy(settings.hardShowTrackAlt, "[[D", sizeof(settings.hardShowTrackAlt));
+        strncpy(settings.showTrackAlt, "", sizeof(settings.showTrackAlt));
         strncpy(settings.hardEndOfPlaylist, "G", sizeof(settings.hardEndOfPlaylist));
         strncpy(settings.hardShowLibrary, "OR", sizeof(settings.hardShowLibrary));
         strncpy(settings.hardShowLibraryAlt, "[[C", sizeof(settings.hardShowLibraryAlt));
+        strncpy(settings.showLibraryAlt, "", sizeof(settings.showLibraryAlt));
         strncpy(settings.hardShowSearch, "[15~", sizeof(settings.hardShowSearch));
         strncpy(settings.hardShowSearchAlt, "[[E", sizeof(settings.hardShowSearchAlt));
+        strncpy(settings.showSearchAlt, "", sizeof(settings.showSearchAlt));
         strncpy(settings.hardNextPage, "[6~", sizeof(settings.hardNextPage));
         strncpy(settings.hardPrevPage, "[5~", sizeof(settings.hardPrevPage));
         strncpy(settings.hardRemove, "[3~", sizeof(settings.hardRemove));
+        strncpy(settings.hardRemove2, "[P", sizeof(settings.hardRemove2));
         strncpy(settings.lastVolume, "100", sizeof(settings.lastVolume));
         strncpy(settings.color, "6", sizeof(settings.color));
         strncpy(settings.artistColor, "6", sizeof(settings.artistColor));
@@ -162,10 +170,6 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.toggleVisualizer, sizeof(settings.toggleVisualizer), "%s", pair->value);
                 }
-                else if (strcmp(stringToLower(pair->key), "togglecovers") == 0)
-                {
-                        snprintf(settings.toggleCovers, sizeof(settings.toggleCovers), "%s", pair->value);
-                }
                 else if (strcmp(stringToLower(pair->key), "toggleascii") == 0)
                 {
                         snprintf(settings.toggleAscii, sizeof(settings.toggleAscii), "%s", pair->value);
@@ -238,6 +242,26 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
                 {
                         snprintf(settings.updateLibrary, sizeof(settings.updateLibrary), "%s", pair->value);
                 }
+                else if (strcmp(stringToLower(pair->key), "showplaylistalt") == 0)
+                {
+                        snprintf(settings.showPlaylistAlt, sizeof(settings.showPlaylistAlt), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "showlibraryalt") == 0)
+                {
+                        snprintf(settings.showLibraryAlt, sizeof(settings.showLibraryAlt), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "showtrackalt") == 0)
+                {
+                        snprintf(settings.showTrackAlt, sizeof(settings.showTrackAlt), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "showsearchalt") == 0)
+                {
+                        snprintf(settings.showSearchAlt, sizeof(settings.showSearchAlt), "%s", pair->value);
+                }
+                else if (strcmp(stringToLower(pair->key), "showkeysalt") == 0)
+                {
+                        snprintf(settings.showKeysAlt, sizeof(settings.showKeysAlt), "%s", pair->value);
+                }
         }
 
         freeKeyValuePairs(pairs, count);
@@ -245,15 +269,24 @@ AppSettings constructAppSettings(KeyValuePair *pairs, int count)
         return settings;
 }
 
-KeyValuePair *readKeyValuePairs(const char *file_path, int *count)
+KeyValuePair *readKeyValuePairs(const char *file_path, int *count, time_t *lastTimeAppRan)
 {
         FILE *file = fopen(file_path, "r");
         if (file == NULL)
         {
-                // fprintf(stderr, "Error opening the settings file.\n");
                 return NULL;
         }
 
+        struct stat file_stat;
+        if (stat(file_path, &file_stat) == -1)
+        {
+                perror("stat");
+                return NULL;
+        }
+
+        // Save the modification time (mtime) of the file
+        *lastTimeAppRan = (file_stat.st_mtime > 0) ? file_stat.st_mtime : file_stat.st_mtim.tv_sec;
+
         KeyValuePair *pairs = NULL;
         int pair_count = 0;
 
@@ -318,13 +351,24 @@ void getConfig(AppSettings *settings)
         char *configdir = getConfigPath();
         char *filepath = NULL;
 
+        // Create the directory if it doesn't exist
+        struct stat st = {0};
+        if (stat(configdir, &st) == -1)
+        {
+                if (mkdir(configdir, 0700) != 0)
+                {
+                        perror("mkdir");
+                        exit(EXIT_FAILURE);
+                }
+        }
+
         size_t filepath_length = strlen(configdir) + strlen("/") + strlen(SETTINGS_FILE) + 1;
         filepath = (char *)malloc(filepath_length);
         strcpy(filepath, configdir);
         strcat(filepath, "/");
         strcat(filepath, SETTINGS_FILE);
 
-        KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count);
+        KeyValuePair *pairs = readKeyValuePairs(filepath, &pair_count, &lastTimeAppRan);
 
         free(filepath);
         *settings = constructAppSettings(pairs, pair_count);
@@ -336,7 +380,7 @@ void getConfig(AppSettings *settings)
         useProfileColors = (settings->useProfileColors[0] == '1');
         hideLogo = (settings->hideLogo[0] == '1');
         hideHelp = (settings->hideHelp[0] == '1');
-        
+
         int temp = atoi(settings->color);
         if (temp >= 0)
                 mainColor = temp;
@@ -407,7 +451,11 @@ void setConfig(AppSettings *settings)
                 hideHelp ? c_strcpy(settings->hideHelp, sizeof(settings->hideHelp), "1") : c_strcpy(settings->hideHelp, sizeof(settings->hideHelp), "0");
 
         sprintf(settings->cacheLibrary, "%d", cacheLibrary);
-        sprintf(settings->lastVolume, "%d", getCurrentVolume());
+
+        int currentVolume = getCurrentVolume();
+        currentVolume = (currentVolume <= 0) ? 10 : currentVolume;
+
+        sprintf(settings->lastVolume, "%d", currentVolume);
 
         // Null-terminate the character arrays
         settings->path[MAXPATHLEN - 1] = '\0';
@@ -463,7 +511,6 @@ void setConfig(AppSettings *settings)
         fprintf(file, "togglePause=%s\n", settings->togglePause);
         fprintf(file, "toggleColorsDerivedFrom=%s\n", settings->toggleColorsDerivedFrom);
         fprintf(file, "toggleVisualizer=%s\n", settings->toggleVisualizer);
-        fprintf(file, "toggleCovers=%s\n", settings->toggleCovers);
         fprintf(file, "toggleAscii=%s\n", settings->toggleAscii);
         fprintf(file, "toggleRepeat=%s\n", settings->toggleRepeat);
         fprintf(file, "toggleShuffle=%s\n", settings->toggleShuffle);
@@ -472,6 +519,12 @@ void setConfig(AppSettings *settings)
         fprintf(file, "savePlaylist=%s\n", settings->savePlaylist);
         fprintf(file, "addToMainPlaylist=%s\n", settings->addToMainPlaylist);
         fprintf(file, "updateLibrary=%s\n", settings->updateLibrary);
+        fprintf(file, "\n# The different main views, normally F2-F6: \n");
+        fprintf(file, "showPlaylistAlt=%s\n", settings->showPlaylistAlt);
+        fprintf(file, "showLibraryAlt=%s\n", settings->showLibraryAlt);
+        fprintf(file, "showTrackAlt=%s\n", settings->showTrackAlt);
+        fprintf(file, "showSearchAlt=%s\n", settings->showSearchAlt);
+        fprintf(file, "showKeysAlt=%s\n\n", settings->showKeysAlt);
 
         fprintf(file, "quit=%s\n\n", settings->quit);
         fprintf(file, "# For special keys use terminal codes: OS, for F4 for instance. This can depend on the terminal.\n");
