@@ -16,8 +16,13 @@ bool shuffleEnabled = false;
 bool skipToNext = false;
 bool seekRequested = false;
 bool paused = false;
+bool stopped = true;
+
+bool hasSilentlySwitched;
+
 float seekPercent = 0.0;
 double seekElapsed;
+
 _Atomic bool EOFReached = false;
 _Atomic bool switchReached = false;
 _Atomic bool readingFrames = false;
@@ -25,7 +30,7 @@ pthread_mutex_t dataSourceMutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_mutex_t switchMutex = PTHREAD_MUTEX_INITIALIZER;
 ma_device device = {0};
 ma_int32 *audioBuffer = NULL;
-
+AudioData audioData;
 int bufSize;
 ma_event switchAudioImpl;
 enum AudioImplementation currentImplementation = NONE;
@@ -54,6 +59,10 @@ int m4aDecoderIndex = -1;
 int opusDecoderIndex = -1;
 int vorbisDecoderIndex = -1;
 
+#ifdef USE_LIBNOTIFY
+NotifyNotification *previous_notification;
+#endif
+
 void logTime(const char *message)
 {
         (void)message;
@@ -692,7 +701,7 @@ int prepareNextDecoder(char *filepath)
         }
         setNextDecoder(decoder);
 
-        if (currentDecoder != NULL)
+        if (currentDecoder != NULL && decoder != NULL)
                 ma_data_source_set_next(currentDecoder, decoder);
 
         return 0;
@@ -987,6 +996,12 @@ void seekPercentage(float percent)
 
 void resumePlayback()
 {
+        // if this was unpaused with no song loaded
+        if (audioData.restart)
+        {
+                audioData.endOfListReached = false;
+        }
+
         if (!ma_device_is_started(&device))
         {
                 ma_device_start(&device);
@@ -994,6 +1009,23 @@ void resumePlayback()
 
         paused = false;
 
+        stopped = false;
+
+        if (appState.currentView != SONG_VIEW)
+        {
+                refresh = true;
+        }
+}
+
+void stopPlayback()
+{
+        if (ma_device_is_started(&device))
+        {
+                ma_device_stop(&device);
+        }
+
+        stopped = true;
+
         if (appState.currentView != SONG_VIEW)
         {
                 refresh = true;
@@ -1025,13 +1057,29 @@ void cleanupPlaybackDevice()
         ma_device_uninit(&device);
 }
 
+void clearCurrentTrack()
+{
+        if (ma_device_is_started(&device))
+        {
+                // Stop the device (which stops playback)
+                ma_device_stop(&device);
+        }
+
+        resetDecoders();
+        resetVorbisDecoders();
+        resetOpusDecoders();
+        resetM4aDecoders();
+
+        ma_data_source_set_next(currentDecoder, NULL);
+}
+
 void togglePausePlayback()
 {
         if (ma_device_is_started(&device))
         {
                 pausePlayback();
         }
-        else if (paused)
+        else if (isPaused() || isStopped())
         {
                 resumePlayback();
         }
@@ -1042,6 +1090,11 @@ bool isPaused()
         return paused;
 }
 
+bool isStopped()
+{
+        return stopped;
+}
+
 pthread_mutex_t deviceMutex = PTHREAD_MUTEX_INITIALIZER;
 pthread_cond_t deviceStopped = PTHREAD_COND_INITIALIZER;
 
@@ -1074,6 +1127,13 @@ bool hasBuiltinDecoder(char *filePath)
                                       strcasecmp(extension, ".mp3") == 0));
 }
 
+void setCurrentFileIndex(AudioData *pAudioData, int index)
+{
+        pthread_mutex_lock(&switchMutex);
+        pAudioData->currentFileIndex = index;
+        pthread_mutex_unlock(&switchMutex);
+}
+
 void activateSwitch(AudioData *pAudioData)
 {
         setSkipToNext(false);
@@ -1088,115 +1148,145 @@ void activateSwitch(AudioData *pAudioData)
         pAudioData->switchFiles = true;
 }
 
-void sanitize_filepath(const char *input, char *sanitized, size_t size)
+bool isValidFilepath(const char *path)
 {
-        size_t i, j = 0;
+    if (path == NULL || strlen(path) == 0 || strlen(path) >= PATH_MAX)
+    {
+        return false;
+    }
 
-        if (input != NULL)
-        {
-                for (i = 0; i < strlen(input) && j < size - 1; ++i)
-                {
-                        if (isalnum((unsigned char)input[i]) || strchr(" :[]()/.,?!-", input[i]))
-                        {
-                                sanitized[j++] = input[i];
-                        }
-                }
-        }
+    // Check if the path can be accessed (exists)
+    if (access(path, F_OK) != 0)
+    {
+        return false;  // Path doesn't exist or is inaccessible
+    }
 
-        sanitized[j] = '\0';
+    return true;  // Valid path that exists
 }
 
-char *removeBlacklistedChars(const char *input, const char *blacklist)
-{
-        if (!input || !blacklist)
-        {
-                return NULL;
-        }
-
-        char *output = calloc(strlen(input) + 1, sizeof(char));
-        if (!output)
-        {
-                perror("Failed to allocate memory");
-                exit(EXIT_FAILURE);
-        }
-
-        const char *in_ptr = input;
-        char *out_ptr = output;
-        while (*in_ptr)
-        {
-                // If the current character is not in the blacklist, copy it to the output
-                if (!strchr(blacklist, *in_ptr))
-                {
-                        *out_ptr++ = *in_ptr;
-                }
-                in_ptr++;
-        }
-
-        return output;
-}
-
-gint64 getLengthInSec(double duration)
+gint64 getLengthInMicroSec(double duration)
 {
         return floor(llround(duration * G_USEC_PER_SEC));
 }
 
-int displaySongNotification(const char *artist, const char *title, const char *cover)
+#ifdef USE_LIBNOTIFY
+void onNotificationClosed(NotifyNotification *notification, gpointer user_data)
 {
-        if (!allowNotifications)
+        (void)user_data;
+
+        g_object_unref(notification);
+        previous_notification = NULL;
+}
+
+void removeBlacklistedChars(const char *input, const char *blacklist, char *output, size_t output_size)
+{
+        if (!input || !blacklist || !output || output_size == 0)
         {
-                return 0;
+                return;
         }
 
-        char sanitized_cover[MAXPATHLEN];
-        const char *blacklist = "&;`|*~<>^()[]{}$\\\"";
-        char *sanitizedArtist = removeBlacklistedChars(artist, blacklist);
-        char *sanitizedTitle = removeBlacklistedChars(title, blacklist);
+        const char *in_ptr = input;
+        char *out_ptr = output;
+        size_t chars_copied = 0;
 
-        if (!sanitizedArtist || !sanitizedTitle)
+        while (*in_ptr && chars_copied < output_size - 1)
         {
-                free(sanitizedArtist);
-                free(sanitizedTitle);
-                return -1;
+                // Copy characters not in blacklist
+                if (!strchr(blacklist, *in_ptr))
+                {
+                        *out_ptr++ = *in_ptr;
+                        chars_copied++;
+                }
+                in_ptr++;
         }
 
-        sanitize_filepath(cover, sanitized_cover, sizeof(sanitized_cover));
+        *out_ptr = '\0';
+}
 
-        char message[MAXPATHLEN + 1024];
-        if (strlen(artist) > 0)
+#define NOTIFICATION_INTERVAL_MICROSECONDS 500000 // 0.5 seconds
+
+struct timeval lastNotificationTime = {0, 0};
+static char sanitizedArtist[512];
+static char sanitizedTitle[512];
+
+int canShowNotification()
+{
+        struct timeval now;
+        gettimeofday(&now, NULL);
+
+        long seconds = now.tv_sec - lastNotificationTime.tv_sec;
+        long microseconds = now.tv_usec - lastNotificationTime.tv_usec;
+        long elapsed = seconds * 1000000 + microseconds; // Total elapsed time in microseconds
+
+        if (elapsed >= NOTIFICATION_INTERVAL_MICROSECONDS)
         {
-                snprintf(message, sizeof(message), "%s - %s", sanitizedArtist, sanitizedTitle);
+                lastNotificationTime = now;
+                return 1;
         }
-        else
-        {
-                snprintf(message, sizeof(message), "%s", sanitizedTitle);
-        }
-
-        char *args[] = {"/usr/bin/notify-send", "-a", "kew", message, "--icon", sanitized_cover, NULL};
-
-        pid_t pid = vfork();
-        if (pid == -1)
-        {
-                perror("vfork");
-                free(sanitizedArtist);
-                free(sanitizedTitle);
-                return -1;
-        }
-        else if (pid == 0)
-        {
-                // Child process
-                execv(args[0], args);
-                // If execv fails
-                perror("execv");
-                _Exit(EXIT_FAILURE);
-        }
-
-        // Parent process
-        free(sanitizedArtist);
-        free(sanitizedTitle);
-
         return 0;
 }
 
+void ensureNonEmpty(char *str)
+{
+        if (str == NULL || str[0] == '\0')
+        {
+                str[0] = ' ';
+                str[1] = '\0';
+        }
+}
+
+int displaySongNotification(const char *artist, const char *title, const char *cover)
+{
+    if (!allowNotifications || !canShowNotification() || !notify_is_initted())
+    {
+        return 0;
+    }
+
+    const char *blacklist = "&;`|*~<>^()[]{}$\\\"";
+
+    removeBlacklistedChars(artist, blacklist, sanitizedArtist, sizeof(sanitizedArtist));
+    removeBlacklistedChars(title, blacklist, sanitizedTitle, sizeof(sanitizedTitle));
+
+    ensureNonEmpty(sanitizedArtist);
+    ensureNonEmpty(sanitizedTitle);
+
+    int coverExists = isValidFilepath(cover);
+
+    if (previous_notification == NULL)
+    {
+        previous_notification = notify_notification_new(
+            sanitizedArtist,
+            sanitizedTitle,
+            coverExists ? cover : NULL
+        );
+
+        g_signal_connect(previous_notification, "closed", G_CALLBACK(onNotificationClosed), NULL);     
+    }
+    else
+    {
+        notify_notification_update(
+            previous_notification,
+            sanitizedArtist,
+            sanitizedTitle,
+            coverExists ? cover : NULL
+        );
+    }
+
+    GError *error = NULL;
+
+    if (!notify_notification_show(previous_notification, &error))
+    {
+        if (error != NULL)
+        {
+            fprintf(stderr, "Failed to show notification: %s\n", error->message);
+            g_error_free(error);
+        }
+    }
+
+    return 0;
+}
+#endif
+
 void executeSwitch(AudioData *pAudioData)
 {
         pAudioData->switchFiles = false;
@@ -1219,28 +1309,84 @@ int getCurrentVolume()
         return soundVolume;
 }
 
+int extractPercentage(char *str)
+{
+        int volume = -1;
+        char *percentSign = strchr(str, '%');
+        if (percentSign != NULL)
+        {
+                // Find the start of the number before the '%'
+                while (percentSign > str && *(percentSign - 1) == ' ')
+                        percentSign--;
+                while (percentSign > str && *(percentSign - 1) >= '0' && *(percentSign - 1) <= '9')
+                        percentSign--;
+
+                volume = atoi(percentSign);
+        }
+        return volume;
+}
+
 int getSystemVolume()
 {
         FILE *fp;
         char command_str[1000];
-        char buf[100];
+        char buf[256];
         int currentVolume = -1;
 
-        // Build the command string
+        // Get the default sink's name
         snprintf(command_str, sizeof(command_str),
-                 "pactl get-sink-volume @DEFAULT_SINK@ | awk 'NR==1{print $5}'");
+                 "pactl info | grep 'Default Sink' | awk '{print $3}'");
 
-        // Execute the command and read the output
         fp = popen(command_str, "r");
         if (fp != NULL)
         {
                 if (fgets(buf, sizeof(buf), fp) != NULL)
                 {
-                        sscanf(buf, "%d", &currentVolume);
+                        buf[strcspn(buf, "\n")] = '\0';
+
+                        snprintf(command_str, sizeof(command_str),
+                                 "pactl get-sink-volume %s", buf);
+
+                        FILE *volume_fp = popen(command_str, "r");
+                        if (volume_fp != NULL)
+                        {
+                                while (fgets(buf, sizeof(buf), volume_fp) != NULL)
+                                {
+                                        int tempVolume = extractPercentage(buf);
+                                        if (tempVolume != -1)
+                                        {
+                                                currentVolume = tempVolume;
+                                                break;
+                                        }
+                                }
+                                pclose(volume_fp);
+                        }
                 }
                 pclose(fp);
         }
 
+        // ALSA
+        if (currentVolume == -1)
+        {
+                snprintf(command_str, sizeof(command_str),
+                         "amixer get Master");
+
+                fp = popen(command_str, "r");
+                if (fp != NULL)
+                {
+                        while (fgets(buf, sizeof(buf), fp) != NULL)
+                        {
+                                int tempVolume = extractPercentage(buf);
+                                if (tempVolume != -1)
+                                {
+                                        currentVolume = tempVolume;
+                                        break;
+                                }
+                        }
+                        pclose(fp);
+                }
+        }
+
         return currentVolume;
 }
 
