@@ -71,7 +71,10 @@ void clearScreen(void);
 
 int readInputSequence(char *seq, size_t seqSize);
 
-// Function to check if the key is a function key
 int isFunctionKey(const char *seq);
 
+void convertControlNotationToAscii(char *input, char *output, size_t size);
+
+void convertAsciiToControlNotation(char input, char *output, size_t size);
+
 #endif
