@@ -3,7 +3,7 @@
 
  utils.c
 
- Utility functions for instance for replacing some standard functions with safer alterantives.
+ Utility functions for instance for replacing some standard functions with safer alternatives.
 
 */
 void c_sleep(int milliseconds)
@@ -343,10 +343,6 @@ void shortenString(char *str, size_t maxLength)
         }
 }
 
-void printBlankSpaces(int numSpaces)
-{
-        for (int i = 0; i < numSpaces; i++)
-        {
-                printf(" ");
-        }
+void printBlankSpaces(int numSpaces) {
+    printf("%*s", numSpaces, " ");
 }
